<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katereplicode-plugin" library="katereplicodeplugin" version="7">
<MenuBar>
 <Menu name="projects"><Text>&amp;Projects</Text>
     <Separator />
     <Action name="katereplicode_run" />
     <Action name="katereplicode_stop" />
 </Menu>
</MenuBar>
</gui>
