#ifdef __cplusplus

#include <QDir>
#include <QJsonObject>
#include <QLineEdit>
#include <QModelIndex>
#include <QObject>
#include <QString>
#include <QTextCursor>
#include <QTimer>
#include <QWidget>

#endif
