<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katesymbolviewerplugin" library="katesymbolviewerplugin" version="3" translationDomain="katesymbolviewer">
  <MenuBar>
    <Menu name="view">
      <text>&amp;Settings</text>
      <Separator/>
      <Action name="view_insert_symbolviewer"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
