<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katexmltools" library="libkatexmltoolsplugin" version="9" translationDomain="katexmltools">
  <MenuBar>
    <Menu name="xml">
      <text>&amp;XML</text>
      <Action name="xml_tool_insert_element"/>
      <Action name="xml_tool_close_element"/>
      <Action name="xml_tool_assign"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
