<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katebuild-plugin" library="katebuildplugin" version="16" translationDomain="katebuild-plugin">
  <MenuBar>
    <Menu name="Build Menubar">
      <text>&amp;Build</text>
      <Action name="select_target"/>
      <Action name="build_selected_target"/>
      <Action name="build_and_run_selected_target" />
      <Action name="compile_current_file" />
      <Action name="stop"/>
      <Separator/>
      <Action name="goto_prev"/>
      <Action name="goto_next"/>
      <Separator/>
      <Action name="focus_build_tab_left"/>
      <Action name="focus_build_tab_right"/>
      <Separator/>
      <Action name="load_targets_cmakefileapi"/>
    </Menu>
  </MenuBar>
  <ToolBar name="mainToolBar" noMerge="1">
      <text>Build Plugin Toolbar</text>
  </ToolBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
