<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katefiletreeplugin" library="katefiletreeplugin" version="10" translationDomain="katefiletree">
  <!-- merge in view -->
  <MenuBar>
    <Menu name="go">
      <text>&amp;Go</text>
      <Action name="filetree_prev_document" group="switch_document"/>
      <Action name="filetree_next_document" group="switch_document"/>
      <Action name="filetree_show_active_document" group="switch_document"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
