<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="tabswitcher" library="tabswitcherplugin" version="3" translationDomain="tabswitcherplugin">
  <MenuBar>
    <Menu name="go" merge="1">
      <text>&amp;Go</text>
      <Action name="view_lru_document_next" group="view_switch_tab"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
