#pragma once

#include <QObject>

class TMPL_Q_OBJECT_CLASS_TMPL : public QObject
{
    Q_OBJECT
public:
    explicit TMPL_Q_OBJECT_CLASS_TMPL(QObject *parent = nullptr);

public Q_SLOTS:

Q_SIGNALS:

private:
};
