<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="lspclient" library="lspclient" version="24" translationDomain="lspclient">
  <MenuBar>
    <Menu name="LSPClient Menubar">
      <text>&amp;LSP Client</text>
      <Action name="lspclient_find_definition"/>
      <Action name="lspclient_find_declaration"/>
      <Action name="lspclient_find_type_definition"/>
      <Action name="lspclient_find_references"/>
      <Action name="lspclient_find_implementations"/>
      <Action name="lspclient_clangd_switchheader"/>
      <Action name="lspclient_rust_analyzer_expand_macro"/>
      <Action name="lspclient_highlight"/>
      <Action name="lspclient_symbol_info"/>
      <Action name="lspclient_goto_workspace_symbol"/>
      <Action name="lspclient_format"/>
      <Action name="lspclient_rename"/>
      <Action name="lspclient_quick_fix"/>
      <Action name="lspclient_code_action"/>
      <Action name="lspclient_expand_selection"/>
      <Action name="lspclient_shrink_selection"/>
      <Separator/>
      <Action name="lspclient_diagnostic_switch"/>
      <Action name="lspclient_close_dynamic"/>
      <Action name="lspclient_suspend_all"/>
      <Separator/>
      <Action name="lspclient_restart_server"/>
      <Action name="lspclient_restart_all"/>
      <Separator/>
      <Menu name="lspclient_more_options" noMerge="1">
        <text>More Options</text>
        <DefineGroup name="lspclient_more_options"/>
        <Action name="lspclient_completion_doc"/>
        <Action name="lspclient_signature_help"/>
        <Action name="lspclient_references_declaration"/>
        <Action name="lspclient_completion_parens"/>
        <Action name="lspclient_auto_hover"/>
        <Action name="lspclient_type_formatting"/>
        <Action name="lspclient_incremental_sync"/>
        <Action name="lspclient_highlight_goto"/>
        <Separator/>
        <Action name="lspclient_diagnostics"/>
        <Action name="lspclient_diagnostics_highlight"/>
        <Action name="lspclient_diagnostics_mark"/>
        <Action name="lspclient_diagnostics_hover"/>
        <Separator/>
        <Action name="lspclient_messages"/>
        <Separator/>
        <Action name="lspclient_clangd_memoryusage"/>
      </Menu>
    </Menu>
  </MenuBar>
  <Menu name="ktexteditor_popup" noMerge="1">
    <Action name="lspclient_code_action"/>
    <Separator/>
    <Action name="lspclient_goto_menu"/>
    <Separator/>
    <Action name="lspclient_find_references"/>
    <Action name="lspclient_rename"/>
    <Separator/>
    <Action name="lspclient_other_menu"/>
  </Menu>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
