class Class1
    def foo1_in_Class1()
        print "foo1_in_Class1"
    end

    class Class2
        def self.name
        end
    end
end

def foo1(param1)
    print "foo1"
end

def no_paren_function
    puts 'aaaaa'
end
