<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katesql" library="katesqlplugin" version="9" translationDomain="katesql">
  <MenuBar>
    <Menu name="SQL">
      <text>&amp;SQL</text>
      <Action name="connection_create"/>
      <Action name="connection_remove"/>
      <Action name="connection_edit"/>
      <Action name="connection_reconnect"/>
      <Separator/>
      <Action name="query_run"/>
      <Action name="query_stop"/>
    </Menu>
  </MenuBar>
  <ToolBar name="SQLConnectionsToolBar">
    <text>SQL Connections Toolbar</text>
    <Action name="connection_chooser"/>
    <Action name="connection_create"/>
    <Action name="connection_remove"/>
    <Action name="connection_edit"/>
    <Action name="connection_reconnect"/>
  </ToolBar>
  <ToolBar name="SQLToolBar">
    <text>SQL Toolbar</text>
    <Action name="query_run"/>
    <Action name="query_stop"/>
  </ToolBar>
  <State name="has_connection_selected">
    <enable>
      <Action name="connection_remove"/>
      <Action name="connection_edit"/>
      <Action name="connection_reconnect"/>
      <Action name="query_run"/>
      <Action name="query_stop"/>
    </enable>
  </State>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
