<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katesnippets" library="katesnippetsplugin" version="4" translationDomain="katesnippetsplugin">
  <MenuBar>
    <Menu name="tools">
      <text>&amp;Tools</text>
      <Action name="tools_create_snippet" group="tools_snippet"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
