<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="formatplugin" library="formatplugin" version="2" translationDomain="formatplugin">
    <MenuBar>
        <Menu name="edit">
            <Action name="format_on_save"/>
            <Action name="format_document"/>
        </Menu>
    </MenuBar>
    <ToolBar name="mainToolBar" noMerge="1">
        <text>Format Plugin Toolbar</text>
    </ToolBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
