<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="katexmlcheck" library="libkatexmlcheckplugin" version="5" translationDomain="katexmlcheck">
  <MenuBar>
    <Menu name="xml">
      <text>&amp;XML</text>
      <Action name="xml_check"/>
    </Menu>
  </MenuBar>
</gui>
<!-- kate: space-indent on; indent-width 4; replace-tabs on; -->
<!-- Should I change version to 6? -->
